#define LGFX_USE_V1
#include <LovyanGFX.hpp>
#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include <FS.h>

int sd_init_flag = 0;
#define SPI_MOSI 2 
#define SPI_MISO 41
#define SPI_SCK 42
#define SD_CS 1 
#define LCD_CS -1
#define LCD_BLK 46
#define I2C_SCL 39
#define I2C_SDA 38

SPIClass SD_SPI;

class LGFX : public lgfx::LGFX_Device
{
    lgfx::Panel_ILI9488 _panel_instance;
    lgfx::Bus_Parallel16 _bus_instance;
  public:
     //Create a constructor and make various settings here.
     // If the class name is changed, the constructor should also specify the same name.
    LGFX(void)
    {
      { //Set bus control.
        auto cfg = _bus_instance.config(); //Get the structure used for bus settings.
        // 16 bit settings
        cfg.port = 0; // Select which I2S port to use (0 or 1) (using ESP32's I2S LCD mode)
        cfg.freq_write = 20000000; // Send clock (up to 20MHz, 80MHz can be rounded to an integer)
        cfg.pin_wr = 18; // Connect the pin number of WR
        cfg.pin_rd = 48; // Connect the pin number of RD
        cfg.pin_rs = 45; // Connect the pin number of RS (D/C)

        cfg.pin_d0 = 47;
        cfg.pin_d1 = 21;
        cfg.pin_d2 = 14;
        cfg.pin_d3 = 13;
        cfg.pin_d4 = 12;
        cfg.pin_d5 = 11;
        cfg.pin_d6 = 10;
        cfg.pin_d7 = 9;
        cfg.pin_d8 = 3;
        cfg.pin_d9 = 8;
        cfg.pin_d10 = 16;
        cfg.pin_d11 = 15;
        cfg.pin_d12 = 7;
        cfg.pin_d13 = 6;
        cfg.pin_d14 = 5;
        cfg.pin_d15 = 4;

        _bus_instance.config(cfg);              // Reflect the set value on the bus.
        _panel_instance.setBus(&_bus_instance);
      }
      { // Set Display Panel Controls
        auto cfg = _panel_instance.config(); // Get the structure used to set the display panel.
        cfg.pin_cs = -1; // CS to be pulled low
        cfg.pin_rst = -1; // RST is associated with the development board RST
        cfg.pin_busy = -1; // Connect the pin number of BUSY (-1=disable)
        //※The settings below set common initial values on each panel
        cfg.memory_width = 320; // The maximum width supported by the driver IC
        cfg.memory_height = 480; // The maximum height supported by the driver IC
        cfg.panel_width = 320; // actual visible width
        cfg.panel_height = 480; // actual visible height
        cfg.offset_x = 0; // Panel X-direction offset
        cfg.offset_y = 0; // Panel Y offset
        cfg.offset_rotation = 0; // Offset of rotation direction value 0~7 (4~7 reverse up and down)
        cfg.dummy_read_pixel = 8; // Number of virtual read bits before pixel read
        cfg.dummy_read_bits = 1; // The number of dummy read bits before reading non-pixel data
        cfg.readable = true; // Set to true when data can be read
        cfg.invert = false; // Set to true if the panel's light and dark are reversed
        cfg.rgb_order = false; // If the red and blue in the panel are replaced with true
        cfg.dlen_16bit = true; // Set to true in the case of a panel that sends data length in units of 16 bits
        cfg.bus_shared = true; // Set to true when sharing the bus with the SD card (bus control via drawJpgFile etc.)
        _panel_instance.config(cfg);
      }
      setPanel(&_panel_instance); // Sets the panel to use.
    }
};

void listDir(fs::FS &fs, const char * dirname, uint8_t levels){
  Serial.printf("Contenu du répertoire : %s\n", dirname);

  File root = fs.open(dirname);
  if(!root){
    Serial.println("Impossible d'ouvrir le répertoire");
    return;
  }
  if(!root.isDirectory()){
    Serial.println("Ce n'est pas un répertoire");
    return;
  }

  File file = root.openNextFile();
  while(file){
    if(file.isDirectory()){
      Serial.print("  DIR : ");
      Serial.println(file.name());
      if(levels){
        listDir(fs, file.name(), levels -1);
      }
    } else {
      Serial.print("  FICHIER: ");
      Serial.print(file.name());
      Serial.print("  TAILLE: ");
      Serial.println(file.size());
    }
    file = root.openNextFile();
  }
}



int SD_init()
{
  SD_SPI.begin(SPI_SCK, SPI_MISO, SPI_MOSI);
  if (!SD.begin(SD_CS, SD_SPI, 40000000))
  {
    Serial.println("Card Mount Failed");
    return 1;
  }
  uint8_t cardType = SD.cardType();

  if (cardType == CARD_NONE)
  {
    Serial.println("No TF card attached");
    return 1;
  }

  uint64_t cardSize = SD.cardSize() / (1024 * 1024);
  Serial.printf("TF Card Size: %lluMB\n", cardSize);
  listDir(SD, "/", 2);
  return 0;
}

LGFX lcd;

int print_img(fs::FS & fs, String filename, int x, int y)
{
  File f = fs.open(filename, "r");
  if (!f)
  {
    Serial.println("Failed to open file for reading");
    f.close();
    return 0;
  }

  f.seek(54);
  int X = x;
  int Y = y;
  uint8_t RGB[3 * X];
  for (int row = 0; row < Y; row++)
  {
    f.seek(54 + 3 * X * row);
    f.read(RGB, 3 * X);
    lcd.pushImage(0, row, X, 1, (lgfx::rgb888_t *)RGB);
  }

  f.close();
  return 0;
}

//显示文件中的图像
//BMP is 32 bit...
int print_img_small(fs::FS & fs, String filename, int x, int y, int w, int h)
{
  File f = fs.open(filename, "r");
  if (!f)
  {
    Serial.println("Failed to open file for reading");
    f.close();
    return 0;
  }

  f.seek(54);
  int X = w;
  int Y = h;
  uint8_t RGB[3 * X];
  uint8_t RGBA[4 * X];
  for (int row = 0; row < Y; row++)
  {
    f.seek(54 + 4 * X * row);
    f.read(RGBA, 4 * X);

    for (int i = 0; i < X; i++)
    {
      RGB[3 * i] = RGBA[4 * X - 4 * i - 4];
      RGB[3 * i + 1] = RGBA[4 * X - 4 * i - 3];
      RGB[3 * i + 2] = RGBA[4 * X - 4 * i - 2];
    }
    lcd.pushImage(x, y + row, X, 1, (lgfx::rgb888_t *)RGB);
  }

  f.close();
  return 0;
}


void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);

  //display screen
  lcd.init();
  //lcd.setSwapBytes(true);
  lcd.fillScreen(TFT_BLACK);
  delay(500);
  //Backlight pin initialization
  pinMode(LCD_BLK, OUTPUT);
  digitalWrite(LCD_BLK, HIGH);
  if (SD_init() == 0)
  {
    Serial.println("TF card initialized successfully");
    lcd.fillScreen(TFT_BLACK);
    lcd.setCursor(115, 230);
    lcd.println("TF card successfully mounted");
  } else {
    Serial.println("TF card initialization failed");
    lcd.fillScreen(TFT_BLACK);
    lcd.setCursor(115, 230);
    lcd.println("TF card failed to mount");
  }
  lcd.printf("Setup done");
  lcd.setRotation(7);
  lcd.fillScreen(TFT_BLACK);
}

void loop() {
  // put your main code here, to run repeatedly:

  print_img(SD, "/1.bmp", 480, 320);//Show SD card photos
  delay(2000);
  print_img(SD, "/2.bmp", 480, 320);
  delay(2000);
  print_img(SD, "/3.bmp", 480, 320);
  delay(2000);

}
